@@ -191,6 +191,7 @@ def configure_network_instances(config_before, config_leftover, interfaces_by_vr
         xe_ospfv2.configure_xe_ospf_redistribution(net_inst, config_before, config_leftover, router_ospf_by_vrf)
         xe_bgp.configure_xe_bgp(net_inst, config_before, config_leftover, network_instances_notes)
         xe_bgp.configure_xe_bgp_redistribution(net_inst, config_before, config_leftover)
+        xe_mpls.configure_xe_mpls(net_inst, config_before, config_leftover, network_instances_notes)
 
 
 def configure_network_interfaces(net_inst, interfaces_by_vrf):
@@ -248,7 +249,8 @@ def configure_pim_network_instance(config_before, config_leftover):
             "openconfig-network-instance:interface-id": "",
             "openconfig-network-instance:mode": "",
             "openconfig-network-instance:dr-priority": 0,
-            "openconfig-network-instance:hello-interval": 0
+            "openconfig-network-instance:hello-interval": 0,
+            "openconfig-pim-ext:neighbor-filter": ""
         },
         "openconfig-network-instance:interface-ref": {
             "openconfig-network-instance:config": {
@@ -278,6 +280,8 @@ def configure_pim_network_instance(config_before, config_leftover):
                         tmp_pim_interface["openconfig-network-instance:config"]["openconfig-network-instance:dr-priority"] = pim_value
                     if "query-interval" in pim_key:
                         tmp_pim_interface["openconfig-network-instance:config"]["openconfig-network-instance:hello-interval"] = pim_value
+                    if "neighbor-filter" in pim_key:
+                        tmp_pim_interface["openconfig-network-instance:config"]["openconfig-pim-ext:neighbor-filter"] = str(pim_value)
                     if "mode" in pim_key:
                         if "sparse-dense-mode" in pim_value:
                             tmp_pim_interface["openconfig-network-instance:config"]["openconfig-network-instance:mode"] = "openconfig-pim-types:PIM_MODE_DENSE"
@@ -679,8 +683,11 @@ def cleanup_null_bgp_leftovers(config_before, config_leftover):
     clean_up_vrf_neighbors_and_peers(bgp_before.get("address-family", {}).get("with-vrf", {}),
                                      bgp_leftover.get("address-family", {}).get("with-vrf", {}).get("ipv4", []))
 
-    if bgp_leftover != None and len(bgp_leftover["bgp"]) == 0:
+    if bgp_leftover != None and bgp_leftover.get("bgp") != None:
         del bgp_leftover["bgp"]
+
+    # if bgp_leftover != None and len(bgp_leftover["bgp"]) == 0:
+    #     del bgp_leftover["bgp"]
     # if bgp_leftover.get("address-family", {}).get("ipv4") != None:
     #     check_delete_protocol_leftovers(bgp_leftover, "ipv4")
     # if bgp_leftover.get("address-family") != None:
@@ -804,12 +811,14 @@ if __name__ == "__main__":
         from package_nso_to_oc.xe import xe_ospfv2
         from package_nso_to_oc.xe import xe_static_route
         from package_nso_to_oc.xe import xe_bgp
+        from package_nso_to_oc.xe import xe_mpls
         from package_nso_to_oc import common
     else:
         import common_xe
         import xe_ospfv2
         import xe_static_route
         import xe_bgp
+        import xe_mpls
         import common
 
     (config_before_dict, config_leftover_dict, interface_ip_dict) = common_xe.init_xe_configs()
@@ -827,10 +836,12 @@ else:
         from package_nso_to_oc.xe import xe_ospfv2
         from package_nso_to_oc.xe import xe_static_route
         from package_nso_to_oc.xe import xe_bgp
+        from package_nso_to_oc.xe import xe_mpls
         from package_nso_to_oc import common
     else:
         from xe import common_xe
         from xe import xe_ospfv2
         from xe import xe_static_route
         from xe import xe_bgp
+        from xe import xe_mpls
         import common
